; ***********************************************************************************
; * Name		:	G. Mayer 														*
; * Customer	:	Gartenhaus Steuerung NG V01										*
; * Datum		:	07.11.2010														*
; * 																				*
; * nderungen	:																	*
; *					V01 Versuchsaufbau auf Lochrasterkarte			GM 07.11.2010	*
; *					V02	Handgetztes Serienprodukt					GM 26.03.2011	*
; *					V03 Steuerung ber RS232 implementiert			GM 28.03.2011	*
; *																					*
; *  2010 KeiLeRSofT																*
; ***********************************************************************************

; ***********************************************************************************
; * Fuses:																			*
; *																					*
; * OCDEN		: 	no																*
; * JTAGEN		:	no																*
; * SPIEN		:	yes																*
; * EESAVE		:	yes																*
; * BOOTSZ		:	Boot Flash size=1024 words Boot address=$0C00					*
; * BOOTRST		:	no																*
; * CKOPT		:	no																*
; * BODLEVEL	:	Brown-out detection at VCC=2.7V									*
; * BODEN		:	yes																*	
; * SUT_CKSEL	:	Ext. Crystal/Resonator High Freq.; Start-up time: 16K CK + 64ms	*
; ***********************************************************************************

.include "m16def.inc"	

; ***********************************************************************************
; * Konstanten																		*
; ***********************************************************************************
.equ K_DatenAnzahlLesen						= 512
.equ K_DatenAnzahlSchreiben					= 32
.equ K_DatenAnzahlDeviceID					= 16
.equ K_Intervall10ms						= 78				; *128s = 10ms
.equ K_Intervall100ms						= 10				; *10ms = 100ms
.equ K_Intervall1000ms						= 10				; *100ms = 1000ms
.equ K_Intervall2500ms						= 250				; *10ms = 2500ms
.equ K_Startzeit							= 1400
.equ K_DataOffsetKorrekturBegrenzung		= 180
.equ K_BitZeit								= 600
.equ K_BitAnzahl							= 16
.equ K_PrellZyclen							= 3
.equ K_AusZyclen							= 3
.equ K_FaktorTimer 							= 8

.equ K_TasteOnOff							= 7
.equ K_Taste1								= 2
.equ K_Taste2								= 1
.equ K_Taste3								= 0
.equ K_Taste4								= 6
.equ K_Taste5								= 5
.equ K_Taste6								= 4
.equ K_Taste7								= 3

.equ K_Relais1								= 0b00000001
.equ K_Relais2								= 0b00000010
.equ K_Relais3								= 0b00000100
.equ K_Relais4								= 0b00001000
.equ K_Relais5								= 0b00010000
.equ K_Relais6								= 0b00100000
.equ K_Relais7								= 0b01000000
.equ K_Fan									= 0b10000000

; ***********************************************************************************
; * Register Zuweisungen															*
; ***********************************************************************************
.def R_FallingTimeL							= R0
.def R_FallingTimeH							= R1
.def R_BitZaehler							= R2
.def R_DataBufferLow						= R3
.def R_DataBufferHigh						= R4
;.def SegmentRotationCount					= R5
;.def SegmentRotationPrescale				= R6
;.def frei 									= R7
;.def Data_Fehler_Taste0					= R8
;.def Data_Fehler_Taste1					= R9
;.def Data_Fehler_Taste2					= R10
;.def Data_Fehler_Taste3					= R11
;.def R_Sekunden					  		= R12
;.def R_Minuten								= R13
;.def R_Stunden	 							= R14
;.def R_TageL								= R15
;.def R_TageH								= R16

.def R_Prescaler100ms 						= R17
.def R_Prescaler1000ms						= R18
.def R_AuszeitTaste							= R19
.def R_PrellzeitTaste	 					= R20
.def R_DataSchwelleL						= R21
.def R_DataSchwelleH						= R22
		
.def R_OUTPUT								= R23
.equ Flag_Relais1							= 0
.equ Flag_Relais2							= 1
.equ Flag_Relais3							= 2
.equ Flag_Relais4							= 3
.equ Flag_Relais5							= 4
.equ Flag_Relais6							= 5
.equ Flag_Relais7							= 6
.equ Flag_Fan								= 7

.def R_STATUS	 							= R24
.equ Flag_10ms 								= 0					
.equ Flag_100ms	 							= 1					
.equ Flag_CodeChange						= 2
.equ Flag_NeueTaste_						= 3
.equ Flag_NeueTasteFunk						= 4
.equ Flag_AkkuWarning						= 5					; Akku Ladezustand schwach
.equ Flag_AkkuLock							= 6					; Akku Ladezustand sehr schwach
.equ Flag_TempWarning						= 7

.def R_STATUSRS232							= R25
.equ Flag_GetData  							= 1					
.equ Flag_SetData							= 2					
.equ Flag_SaveData							= 3
.equ Flag_DeleteData						= 4
.equ Flag_Remote							= 5
.equ Flag_FanControlInaktive				= 6

;ab	R26-R31 belegt fr XL,XH,YL,YH,ZL,ZH

; ***********************************************************************************
; * RAM fr aktuellen Wert															*
; ***********************************************************************************
.dseg
RAM_TastenBufferNeu						:	.byte 1
RAM_TastenBufferAlt						:	.byte 1
RAM_PointerScanHigh						:	.byte 1
RAM_PointerScanLow						:	.byte 1

RAM_HerstellerCode						:	.byte 1
RAM_FunkCodeTaste1						:	.byte 1
RAM_FunkCodeTaste2						:	.byte 1
RAM_FunkCodeTaste3						:	.byte 1
RAM_FunkCodeTaste4						:	.byte 1
RAM_FunkCodeTaste5						:	.byte 1
RAM_FunkCodeTaste6						:	.byte 1
RAM_FunkCodeTaste7						:	.byte 1

RAM_TemperaturSchwellwertLow			:	.byte 1										; Lfter Aus
RAM_TemperaturSchwellwertHigh			:	.byte 1										; Lfert An
RAM_TemperaturSchwellwertHot			:	.byte 1										; 24V Ausgnge aus, Lfter bleibt An
RAM_TemperaturSchwellwertOverHead		:	.byte 1										; Gert Aus

RAM_AkkuSchwellwertNoEnergy				:	.byte 1
RAM_AkkuSchwellwertLowEnergyRed			:	.byte 1
RAM_AkkuSchwellwertLowEnergyYellow		:	.byte 1
RAM_AkkuSchwellwertHighEnergy			:	.byte 1

RAM_Reserviert							:	.byte 12

RAM_DatenBuffer							:	.byte K_DatenAnzahlSchreiben
RAM_DatenBufferZeiger					:	.byte 2

; ***********************************************************************************
; * EEPROM Area																		*
; ***********************************************************************************
.eseg
EE_RWDaten								:	.db	0xD4,0xF6,0x63,0xEB,0xA3,0xD7,0x89,0x55	; HerstellerCode, Code 1 - 7
											.db 0x94,0x77,0x69,0x61,0x88,0x99,0xA9,0xC2 ; Temp Fan Off, Temp Fan On, Temp Hot, Temp OverHead, AKKU OFF, AKKU Rot, AKKU Gelb, AKKU OK
											.db 0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
											.db 0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF

											.db 0x47,0x48,0x53,0x2D,0x4E,0x47,0x20,0x56 ; Gerte ID -> GHS-NGV01
											.db 0x30,0x31,0xFF,0xFF,0xFF,0xFF,0xFF,0xFF
											
; ab EEprom Position 49
EE_MaxTemperatur						:	.db 0xFF
EE_AktuelleTemperatur					:	.db 0xFF
EE_MinAkkuSpannung						:	.db 0xFF
EE_AktuelleAKKUSpannung					:	.db 0xFF
;EE_Minuten								:	.db 0xFF
;EE_Stunden								:	.db 0xFF
;EE_TageL								:	.db 0xFF
;EE_TageH								:	.db 0xFF
EE_PortStatus							:	.db 0xFF

; ***********************************************************************************
; * Port Konstanten Port A (Mega16)													*
; ***********************************************************************************
.equ	AD_Temperatur						= 0
;.equ	AD1									= 1
;.equ	AD2									= 2
;.equ	AD3									= 3
;.equ	AD4									= 4
;.equ	AD5									= 5
.equ	AD_AKKU								= 6
.equ	OnOffPin							= PinA
.equ	OnOffPort							= PortA
.equ	OnOff								= 7

; ***********************************************************************************
; * Port Konstanten Port B															*
; ***********************************************************************************
.equ	StatusLEDYellowPort					= PortB
.equ	StatusLEDYellow						= 0
.equ	StatusLEDRedPort					= PortB
.equ	StatusLEDRed						= 1
.equ	Taste3Port							= PinB
.equ	Taste3								= 2
.equ	Taste2Port							= PinB
.equ	Taste2								= 3
.equ	Taste1Port							= PinB
.equ	Taste1								= 4
;.equ	MOSI								= 5
;.equ	MISO								= 6
;.equ	SCK									= 7

; ***********************************************************************************
; * Port Konstanten Port C (AD Ports)												*
; ***********************************************************************************
;.equ	TestPort							= PortC
;.equ	Test								= 0

; ***********************************************************************************
; * Port Konstanten Port C															*
; ***********************************************************************************
.equ	OutputPin							= PinC
.equ	OutputPort							= PortC
.equ	Relais1Port							= PortC
.equ	Relais1								= 0
.equ	Relais2Port							= PortC
.equ	Relais2								= 1
.equ	Relais3Port							= PortC
.equ	Relais3								= 2
.equ	Relais4Port							= PortC
.equ	Relais4								= 3
.equ	Relais5Port							= PortC
.equ	Relais5								= 4
.equ	Relais6Port							= PortC
.equ	Relais6								= 5
.equ	Relais7Port							= PortC
.equ	Relais7								= 6
.equ	LuefterPort							= PortC
.equ	Luefter								= 7

; ***********************************************************************************
; * Port Konstanten Port D															*
; ***********************************************************************************
;.equ	RxD									= 0
;.equ	TxD									= 1
.equ	DataInPort							= PinD
.equ	DataIn								= 2
.equ	Taste7Port							= PinD
.equ	Taste7								= 3
.equ	Taste6Port							= PinD
.equ	Taste6								= 4
.equ	Taste5Port							= PinD
.equ	Taste5								= 5
.equ	Taste4Port							= PinD
.equ	Taste4								= 6
.equ	TasterOnOffPort						= PinD
.equ	TasterOnOff							= 7

; ***********************************************************************************
; * Initialization																	*
; ***********************************************************************************
.cseg
	jmp 	RESET 											; Reset Handler
	jmp 	EXT_INT0 										; IRQ0 Handler
	jmp 	EXT_INT1 										; IRQ1 Handler
	jmp 	TIM2_COMP 										; Timer2 Compare Handler
	jmp 	TIM2_OVF 										; Timer2 Overflow Handler
	jmp		TIM1_CAPT 										; Timer1 Capture Handler
	jmp 	TIM1_COMPA_INT 									; Timer1 Compare A Handler
	jmp 	TIM1_COMPB_INT 									; Timer1 Compare B Handler
	jmp 	TIM1_OVF 										; Timer1 Overflow Handler
	jmp 	TIM0_OVF 										; Timer0 Overflow Handler
	jmp 	SPI_STC 										; SPI Transfer Complete Handler
	jmp 	USART_RXC 										; USART, RX Complete Handler
	jmp 	USART_UDRE 										; USART, UDR Empty Handler
	jmp 	USART_TXC		 								; USART, TX Complete Handler
	jmp 	ADC_INT 										; ADC Conversion Complete Handler
	jmp 	EE_RDY 											; EEPROM Ready Handler
	jmp 	ANA_COMP 										; Analog Comparator Handler
	jmp 	TWI 											; 2-wire Serial Interface Handler
	jmp		EXT_INT2										; IRQ2 Handler						(Mega16)
	jmp		TIM0_COMP										; Timer0 Compare Handler			(Mega16)
	jmp 	SPM_RDY											; Store Program Memory Ready

; ***********************************************************************************
; * Reset Handler																	*
; ***********************************************************************************
RESET:
	ldi 	XL,Low(RAMEND)									; Main program start
	out 	SPL,XL	 										; Set Stack Pointer to top of RAM
	ldi 	XL,High(RAMEND)									; Main program start
	out 	SPH,XL 											; Set Stack Pointer to top of RAM
	wdr														; reset watchdog

	; *******************************************************************************
	; * USART Init																	*
	; *******************************************************************************
	ldi		XL,0
	out		UBRRH,XL
	ldi		XL,12 											; 25 -> 38,4 bei 16 MHz / 119 -> 9600 bei 18,4320 MHz
	out		UBRRL,XL
	ldi		XL,(1<<RXCIE)|(1<<RXEN)|(1<<TXEN)
	out		UCSRB,XL
	ldi		XL,(1<<URSEL)|(0<<USBS)|(3<<UCSZ0)
	out		UCSRC,XL

	wdr														; reset watchdog
	ldi		XL,0b00011000									; watchdog change enable
	out		WDTCR,XL
	ldi		XL,0b00011011									; watchdog prescal set to 125ms
	out		WDTCR,XL
	ldi		XL,0b00001011									; watchdog change disable
	out		WDTCR,XL
	wdr

; ***********************************************************************************
; * Port Deklarationen Port A	(Mega16)											*
; ***********************************************************************************
	ldi		XL,0b00000000		; PORTA						; PullUp 	: 0 -> Nein	, 1 -> Ja
	ldi		XH,0b10000000		; DDRA	 					; Richtung 	: 0 -> Input, 1 -> Output
;                ||||||||
;                |||||||+- 0      - 						; (input) 			
;                ||||||+-- 1      - 						; (input)  													
;                |||||+--- 2      - 						; (input)  						
;                ||||+---- 3      - 						; (input) 													
;                |||+----- 4      - 						; (input) 
;                ||+------ 5      - 						; (input)	
;                |+------- 6      - 						; (input) 		  
;                +-------- 7      OnOff (Selbsthaltung)		; (output) 		 									
	out		PortA,XL
	out		DDRA,XH

; ***********************************************************************************
; * Port Deklarationen Port B														*
; ***********************************************************************************
	ldi		XL,0b00011111		; PORTB						; PullUp 	: 0 -> Nein	, 1 -> Ja
	ldi		XH,0b00000011		; DDRB	 					; Richtung 	: 0 -> Input, 1 -> Output
;                ||||||||
;                |||||||+- 0      - Status LED gelb 		; (output) 			
;                ||||||+-- 1      - Status LED rot 			; (output)  													
;                |||||+--- 2      - Taster3					; (input PullUp)  						
;                ||||+---- 3      - Taster3					; (input PullUp) 													
;                |||+----- 4      - Taster3 				; (input PullUp) 
;                ||+------ 5      - MOSI	 				; (output)	
;                |+------- 6      - MISO					; (input) 		  
;                +-------- 7      - SCK						; (input) 		 									
	out		PortB,XL
	out		DDRB,XH

; ***********************************************************************************
; * Port Deklarationen Port C														*
; ***********************************************************************************
	ldi		XL,0b00000000		; PORTC						; PullUp 	: 0 -> Nein	, 1 -> Ja
	ldi		XH,0b11111111		; DDRC	 					; Richtung 	: 0 -> Input, 1 -> Output
;                ||||||||
;                |||||||+- 0    - Relais7					; (output)	
;                ||||||+-- 1    - Relais6					; (output)
;                |||||+--- 2    - Relais5					; (output)
;                ||||+---- 3    - Relais4					; (output)
;                |||+----- 4    - Relais3					; (output)
;                ||+------ 5    - Relais2					; (output)
;                |+------- 6    - Relais1					; (output)
;                +-------- 7	- Lfter					; (output)
	out		PortC,XL
	out		DDRC,XH

; ***********************************************************************************
; * Port Deklarationen Port D														*
; ***********************************************************************************
	ldi		XL,0b11111000		;PORTD						; PullUp 	: 0 -> Nein	, 1 -> Ja
	ldi		XH,0b00000000		;DDRD	 					; Richtung 	: 0 -> Input, 1 -> Output
;                ||||||||
;                |||||||+- 0  	- RxD						; (input) 		 															 
;                ||||||+-- 1    - TxD						; (input) 		 										
;                |||||+--- 2    - Funk Data					; (input) 			 		
;                ||||+---- 3    - Taster7					; (input PullUp) 		 
;                |||+----- 4    - Taster6					; (input PullUp) 		 		
;                ||+------ 5    - Taster5					; (input PullUp) 		 						
;                |+------- 6    - Taster4					; (input PullUp) 		 
;                +-------- 7    - OnOff						; (input PullUp) 		 
	out		PortD,XL
	out		DDRD,XH

; ***********************************************************************************
; * MCUCR Config																	*
; ***********************************************************************************
;ldi		XL,0b10000001		; sleep enable, INT0 any logical change (Mega8)
	ldi		XL,0b01000001		; sleep enable, INT0 any logical change (Mega16)
;                ||||||||
;                |||||||+- 0    - ISC00
;                ||||||+-- 1    - ISC01
;                |||||+--- 2    - ISC10
;                ||||+---- 3    - ISC11
;                |||+----- 4    - SM0
;                ||+------ 5    - SM1
;                |+------- 6    - SM2
;                +-------- 7    - SE
	out		MCUCR,XL			; Sleepmode Enable


; ***********************************************************************************
; * Clock Select fr Timer2 / 64 = 1Tick 8s										*
; ***********************************************************************************
	ldi		XL,0b00000111
;                ||||||||
;                |||||||+- 0    - CS00
;                ||||||+-- 1    - CS01
;                |||||+--- 2    - CS02
;                ||||+---- 3    - CTC0
;                |||+----- 4    - unused
;                ||+------ 5    - unused
;                |+------- 6    - unused
;                +-------- 7    - unused
	out		TCCR2,XL

; ***********************************************************************************
; * Timer2 auf CompareB vorbereiten fr Funkscann									*
; ***********************************************************************************
	in		XL,TIMSK
	ori 	XL,0b10000000
;                ||||||||
;                |||||||+- 0    - TOIE0
;                ||||||+-- 1    - OCIE0 (Mega16)	
;                |||||+--- 2    - TOIE1
;                ||||+---- 3    - OCIE1B
;                |||+----- 4    - OCIE1A
;                ||+------ 5    - TICIE1
;                |+------- 6    - TOIE2
;                +-------- 7    - OCIE2
	out		TIMSK,XL

; ***********************************************************************************
; * TCCR1B Config -> Timer 1/64														*
; ***********************************************************************************
	ldi		XL,0b00000011
;                ||||||||
;                |||||||+- 0    - CS10
;                ||||||+-- 1    - CS11
;                |||||+--- 2    - CS12
;                ||||+---- 3    - WGM12
;                |||+----- 4    - WGM13
;                ||+------ 5    - unused
;                |+------- 6    - ICES1
;                +-------- 7    - ICNC1
	out		TCCR1B,XL		

; ***********************************************************************************
; * GICR Config -> Timer 1/64														*
; ***********************************************************************************
	ldi		XL,0b01000000			
;                ||||||||
;                |||||||+- 0    - IVCE
;                ||||||+-- 1    - IVSEL
;                |||||+--- 2    - unused
;                ||||+---- 3    - unused
;                |||+----- 4    - unused
;                ||+------ 5    - INT2
;                |+------- 6    - INT0
;                +-------- 7    - INT1	
	out		GICR,XL												; enable int0 interrupt	

; ***********************************************************************************
; * Initialisierungen																*
; ***********************************************************************************
Init:
;	sbi		OnOffPort,OnOff
	clr		R_STATUS
	clr		R_STATUSRS232
	clr		R_OUTPUT
	ldi		R_Prescaler100ms,K_Intervall100ms					; Beim ersten mal nach dem Einschalten 2,5s warten
	ldi		R_Prescaler1000ms,1									; Beim ersten mal nach dem Einschalten 0s warten
	clr		R_PrellzeitTaste
	ldi		R_AuszeitTaste,K_AusZyclen

	ldi		XL,255
	sts		RAM_TastenBufferAlt,XL

	rcall	EEpromToRAM											; Daten aus dem EEprom ins RAM holen (Pos 0-32)
	sbis	Taste1Port,Taste1
	sbr		R_STATUSRS232,(1<<Flag_DeleteData)

InitEnd:
	sei															; enable all interrupts

; ***********************************************************************************
; * Hauptprogramm																	*
; ***********************************************************************************
MainLoop:
	wdr
	sleep

	sbrs	R_STATUS,Flag_10ms									; kommt alle 100ms
	rjmp	MainLoop

	; *********************************** 
	; * 100ms Task						*
	; *********************************** 
	cbr		R_STATUS,(1<<Flag_10ms) 							; clear Flag		
	sbi		StatusLEDRedPort,StatusLEDRed						; StatusLED Rot aus	
	sbi		StatusLEDYellowPort,StatusLEDYellow					; StatusLED Gelb aus	
								
	dec		R_Prescaler100ms
	brne	MainLoop

	ldi		R_Prescaler100ms,K_Intervall100ms

	sbrs	R_STATUSRS232,Flag_DeleteData
	rjmp	MainLoop_NotDeleteEErom

	; *********************************** 
	; * EEprom Delete					*
	; *********************************** 
	cbi		StatusLEDRedPort,StatusLEDRed
	rcall	EEpromDelete
	cbr		R_STATUSRS232,(1<<Flag_DeleteData)
	sbi		StatusLEDRedPort,StatusLEDRed
	ldi		R_Prescaler100ms,K_Intervall2500ms					; 2,5s warten
	rjmp	MainLoop

MainLoop_NotDeleteEErom:
	rcall	CheckTemperatur										; Temperatur berwachung
	rcall	CheckAkkuSpannung									; Akku Spannung berwachen
	rcall	TastenAuswertung									; Tasten Auswerten
	rcall	SetOutput
	rcall	SendData											; Daten aus EEprom an PC senden
	rcall	SaveDataToEEprom									; Eingegangene Daten im EEprom speichern

	dec		R_Prescaler1000ms
	brne	MainLoop_End

	; *********************************** 
	; * 1000ms Task						*
	; *********************************** 
	ldi		R_Prescaler1000ms,K_Intervall1000ms					; Prescaler neu laden

	sbrc	R_STATUS,Flag_AkkuWarning
	cbi		StatusLEDYellowPort,StatusLEDYellow					; StatusLED Gelb Ein

	sbrc	R_STATUS,Flag_AkkuLock
	cbi		StatusLEDRedPort,StatusLEDRed						; StatusLED Rot Ein

MainLoop_End:
	rjmp	MainLoop

; ***********************************************************************************
; * Unterrotienen	     															*
; ***********************************************************************************

; ***********************************************************************************
; * Tastenauswertung																*
; ***********************************************************************************
TastenAuswertung:
	in		XL,PinB
	ror		XL
	ror		XL
	andi	XL,0b00000111
	in		XH,PinD
	andi	XH,0b11111000
	or		XL,XH

	; *******************************************************************************
	; * Taste akzeptieren (Mainboard)												*
	; *******************************************************************************
	sbrs	XL,K_TasteOnOff
	rjmp	TastenAuswertung_OnOff

	sbrs	XL,K_Taste1
	rjmp	TastenAuswertung_Taste1

	sbrs	XL,K_Taste2
	rjmp	TastenAuswertung_Taste2

	sbrs	XL,K_Taste3
	rjmp	TastenAuswertung_Taste3

	sbrs	XL,K_Taste4
	rjmp	TastenAuswertung_Taste4

	sbrs	XL,K_Taste5
	rjmp	TastenAuswertung_Taste5

	sbrs	XL,K_Taste6
	rjmp	TastenAuswertung_Taste6

	sbrs	XL,K_Taste7
	rjmp	TastenAuswertung_Taste7

	sbrs	R_STATUS,Flag_NeueTasteFunk
	rjmp	TastenAuswertung_KeineTaste

	cbr		R_STATUS,(1<<Flag_NeueTasteFunk) 						; Flag lschen
	lds		XL,RAM_TastenBufferNeu
	lds		XH,RAM_TastenBufferAlt
	sts		RAM_TastenBufferAlt,XL
	cpse	XL,XH
	rjmp	TastenAuswertung_KeineTaste

	; *******************************************************************************
	; * Taste akzeptieren (Funk)													*
	; *******************************************************************************
	lds		XH,RAM_FunkCodeTaste1
	cp		XL,XH
	brne	TastenAuswertung_NichtFunkCodeTaste1
	rjmp	TastenAuswertung_Taste1

TastenAuswertung_NichtFunkCodeTaste1:
	lds		XH,RAM_FunkCodeTaste2
	cp		XL,XH
	brne	TastenAuswertung_NichtFunkCodeTaste2
	rjmp	TastenAuswertung_Taste2

TastenAuswertung_NichtFunkCodeTaste2:
	lds		XH,RAM_FunkCodeTaste3
	cp		XL,XH
	brne	TastenAuswertung_NichtFunkCodeTaste3
	rjmp	TastenAuswertung_Taste3

TastenAuswertung_NichtFunkCodeTaste3:
	lds		XH,RAM_FunkCodeTaste4
	cp		XL,XH
	brne	TastenAuswertung_NichtFunkCodeTaste4
	rjmp	TastenAuswertung_Taste4

TastenAuswertung_NichtFunkCodeTaste4:
	lds		XH,RAM_FunkCodeTaste5
	cp		XL,XH
	brne	TastenAuswertung_NichtFunkCodeTaste5
	rjmp	TastenAuswertung_Taste5

TastenAuswertung_NichtFunkCodeTaste5:
	lds		XH,RAM_FunkCodeTaste6
	cp		XL,XH
	brne	TastenAuswertung_NichtFunkCodeTaste6
	rjmp	TastenAuswertung_Taste6

TastenAuswertung_NichtFunkCodeTaste6:
	lds		XH,RAM_FunkCodeTaste7
	cp		XL,XH
	brne	TastenAuswertung_NichtFunkCodeTaste7
	rjmp	TastenAuswertung_Taste7

TastenAuswertung_NichtFunkCodeTaste7:
	rjmp	TastenAuswertung_KeineTaste	

	; *******************************************************************************
	; * Taste OnOff																	*
	; *******************************************************************************
TastenAuswertung_OnOff:
	inc		R_PrellzeitTaste
	cpi		R_PrellzeitTaste,K_PrellZyclen
	brlo	TastenAuswertung_AuszeitNeuLaden_Link
	cpi		R_PrellzeitTaste,K_PrellZyclen
	breq	TastenAuswertung_OnOffToggle
	ldi		R_PrellzeitTaste,K_PrellZyclen+1
	rjmp	TastenAuswertung_AuszeitNeuLaden_Link

TastenAuswertung_OnOffToggle:
	sbic	OnOffPin,OnOff
	rjmp	TastenAuswertung_OnOffToggleOff
	
	sbi		OnOffPort,OnOff
	rjmp	TastenAuswertung_AuszeitNeuLaden_Link

TastenAuswertung_OnOffToggleOff:
	cbi		OnOffPort,OnOff
	rjmp	TastenAuswertung_AuszeitNeuLaden_Link

TastenAuswertung_AuszeitNeuLaden_Link:
	rjmp	TastenAuswertung_AuszeitNeuLaden

	; *******************************************************************************
	; * Taste 1																		*
	; *******************************************************************************
TastenAuswertung_Taste1:
	inc		R_PrellzeitTaste
	cpi		R_PrellzeitTaste,K_PrellZyclen
	brlo	TastenAuswertung_AuszeitNeuLaden
	cpi		R_PrellzeitTaste,K_PrellZyclen
	breq	TastenAuswertung_Taste1Toggle
	ldi		R_PrellzeitTaste,K_PrellZyclen+1
	rjmp	TastenAuswertung_AuszeitNeuLaden

TastenAuswertung_Taste1Toggle:
;	rcall	ToggleRelais1
	rcall	ToggleOutput0
	rjmp	TastenAuswertung_AuszeitNeuLaden

	; *******************************************************************************
	; * Taste 2																		*
	; *******************************************************************************
TastenAuswertung_Taste2:
	inc		R_PrellzeitTaste
	cpi		R_PrellzeitTaste,K_PrellZyclen
	brlo	TastenAuswertung_AuszeitNeuLaden
	cpi		R_PrellzeitTaste,K_PrellZyclen
	breq	TastenAuswertung_Taste2Toggle
	ldi		R_PrellzeitTaste,K_PrellZyclen+1
	rjmp	TastenAuswertung_AuszeitNeuLaden

TastenAuswertung_Taste2Toggle:
;	rcall	ToggleRelais2
	rcall	ToggleOutput1
	rjmp	TastenAuswertung_AuszeitNeuLaden

	; *******************************************************************************
	; * Taste 3																		*
	; *******************************************************************************
TastenAuswertung_Taste3:
	inc		R_PrellzeitTaste
	cpi		R_PrellzeitTaste,K_PrellZyclen
	brlo	TastenAuswertung_AuszeitNeuLaden
	cpi		R_PrellzeitTaste,K_PrellZyclen
	breq	TastenAuswertung_Taste3Toggle
	ldi		R_PrellzeitTaste,K_PrellZyclen+1
	rjmp	TastenAuswertung_AuszeitNeuLaden

TastenAuswertung_Taste3Toggle:
;	rcall	ToggleRelais3
	rcall	ToggleOutput2
	rjmp	TastenAuswertung_AuszeitNeuLaden

	; *******************************************************************************
	; * Taste 4																		*
	; *******************************************************************************
TastenAuswertung_Taste4:
	inc		R_PrellzeitTaste
	cpi		R_PrellzeitTaste,K_PrellZyclen
	brlo	TastenAuswertung_AuszeitNeuLaden
	cpi		R_PrellzeitTaste,K_PrellZyclen
	breq	TastenAuswertung_Taste4Toggle
	ldi		R_PrellzeitTaste,K_PrellZyclen+1
	rjmp	TastenAuswertung_AuszeitNeuLaden

TastenAuswertung_Taste4Toggle:
;	rcall	ToggleRelais4
	rcall	ToggleOutput3
	rjmp	TastenAuswertung_AuszeitNeuLaden

	; *******************************************************************************
	; * Taste 5																		*
	; *******************************************************************************
TastenAuswertung_Taste5:
	inc		R_PrellzeitTaste
	cpi		R_PrellzeitTaste,K_PrellZyclen
	brlo	TastenAuswertung_AuszeitNeuLaden
	cpi		R_PrellzeitTaste,K_PrellZyclen
	breq	TastenAuswertung_Taste5Toggle
	ldi		R_PrellzeitTaste,K_PrellZyclen+1
	rjmp	TastenAuswertung_AuszeitNeuLaden

TastenAuswertung_Taste5Toggle:
;	rcall	ToggleRelais5
	rcall	ToggleOutput4
	rjmp	TastenAuswertung_AuszeitNeuLaden

	; *******************************************************************************
	; * Taste 6																		*
	; *******************************************************************************
TastenAuswertung_Taste6:
	inc		R_PrellzeitTaste
	cpi		R_PrellzeitTaste,K_PrellZyclen
	brlo	TastenAuswertung_AuszeitNeuLaden
	cpi		R_PrellzeitTaste,K_PrellZyclen
	breq	TastenAuswertung_Taste6Toggle
	ldi		R_PrellzeitTaste,K_PrellZyclen+1
	rjmp	TastenAuswertung_AuszeitNeuLaden

TastenAuswertung_Taste6Toggle:
;	rcall	ToggleOutput6
	rcall	ToggleOutput5
	rjmp	TastenAuswertung_AuszeitNeuLaden

	; *******************************************************************************
	; * Taste 7																		*
	; *******************************************************************************
TastenAuswertung_Taste7:
	inc		R_PrellzeitTaste
	cpi		R_PrellzeitTaste,K_PrellZyclen
	brlo	TastenAuswertung_AuszeitNeuLaden
	cpi		R_PrellzeitTaste,K_PrellZyclen
	breq	TastenAuswertung_Taste7Toggle
	ldi		R_PrellzeitTaste,K_PrellZyclen+1
	rjmp	TastenAuswertung_AuszeitNeuLaden

TastenAuswertung_Taste7Toggle:
;	rcall	ToggleRelais7
	rcall	ToggleOutput6
	rjmp	TastenAuswertung_AuszeitNeuLaden

	; *******************************************************************************
	; * Tasten Auszeit neu laden													*
	; *******************************************************************************
TastenAuswertung_AuszeitNeuLaden:
	ldi		R_AuszeitTaste,K_AusZyclen
	rjmp	TastenAuswertungEnd

TastenAuswertung_KeineTaste:
	; *******************************************************************************
	; * Keine Taste																	*
	; *******************************************************************************
	cpi		R_AuszeitTaste,0
	breq	TastenAuswertungAuszeitIstNull
	dec		R_AuszeitTaste
	breq	TastenAuswertungAuszeitIstNull
	rjmp	TastenAuswertungEnd

TastenAuswertungAuszeitIstNull:
	clr		R_PrellzeitTaste

TastenAuswertungEnd:
	ret

; ***********************************************************************************
; * Daten aus EEprom laden															*
; ***********************************************************************************
EEpromToRAM:
	cli															; alle Interrupts Disabled
	ldi		XH,K_DatenAnzahlSchreiben
	clr		YL
	clr		YH
	ldi		ZL,Low(RAM_HerstellerCode)
	ldi		ZH,High(RAM_HerstellerCode)

EEpromToRAMLoop:
	rcall	EEPROM_Read
	adiw	YH:YL,1
	st		Z+,XL
	dec		XH
	brne	EEpromToRAMLoop

	sei															; alle Interruppts Enabled
	ret

; ***********************************************************************************
; * Daten in EEprom lschen (nur nicht Programmrelevante)							*
; ***********************************************************************************
EEpromDelete:
	cli	
	ldi		YL,Low(EE_MaxTemperatur)
	ldi		YH,High(EE_MaxTemperatur)

EEpromDelete_Loop:
	wdr
	ldi		XL,0xFF
	rcall	EEPROM_Write
	adiw	YH:YL,1
	ldi		XH,High(EE_MaxTemperatur+(K_DatenAnzahlLesen-(K_DatenAnzahlSchreiben+K_DatenAnzahlDeviceID)))
	cpi		YL,Low(EE_MaxTemperatur+(K_DatenAnzahlLesen-(K_DatenAnzahlSchreiben+K_DatenAnzahlDeviceID)))
	cpc		YH,XH
	brne	EEpromDelete_Loop

	sei	
	ret

; ***********************************************************************************
; * Analog werte einlesen 															*
; * 												                             	*
; * MUX 											                             	*
; * 	0000	ADC0	Temperatur	 				                             	*
; * 	0001	ADC1									                            *
; * 	0010	ADC2								                           		*
; * 	0011	ADC3								                             	*
; * 	0100	ADC4								                             	*
; * 	0101	ADC5	 							                             	*
; * 	0110	ADC6	AKKU						                             	*
; * 	0111	ADC7		 		 				                             	*
; * 												                             	*
; ***********************************************************************************
; ***********************************************************************************
; * Check Temperatur im Gehuse														*
; ***********************************************************************************
CheckTemperatur:
	sbrc	R_STATUSRS232,Flag_FanControlInaktive				; Wird Lfter ber RS232 gesteuert
	rjmp	CheckTemperatur_End 								; Check Temperatur berspringen

	ldi		XL,AD_Temperatur									; Wahl Analog Channel 
	rcall	Read_Analog

	lds		XH,RAM_TemperaturSchwellwertHigh
	cp		XL,XH
	brsh	CheckTemperatur_LuefterAus

	; *******************************************************************************
	; * Temperatur Obergranze berschritten - Lfter an								*
	; *******************************************************************************
	sbr		R_OUTPUT,K_Fan									; Lfter einschalten

	lds		XH,RAM_TemperaturSchwellwertHot
	cp		XL,XH
	brsh	CheckTemperatur_Save

	; *******************************************************************************
	; * 24V DC-Converter abschalten													*
	; *******************************************************************************
	in		R_OUTPUT,OutputPort
	andi	R_OUTPUT,0b10011111
	out		OutputPort,R_OUTPUT
	sbr		R_STATUS,(1<<Flag_TempWarning)		

	lds		XH,RAM_TemperaturSchwellwertOverHead
	cp		XL,XH
	brsh	CheckTemperatur_Save

	; *******************************************************************************
	; * Gert komplett abschalten													*
	; *******************************************************************************
	cbi		OnOffPort,OnOff
	rjmp	CheckTemperatur_End

	; *******************************************************************************
	; * Temperatur speichern														*
	; *******************************************************************************
CheckTemperatur_Save:
	mov		XH,XL
	ldi		YL,Low(EE_MaxTemperatur)							; Letzte berhhte Temp lesen und
	ldi		YH,High(EE_MaxTemperatur)
	rcall	EEPROM_Read						
	cp		XH,XL												; mit aktueller vergleichen
	brsh	CheckTemperatur_End									; ist die aktuelle grer der gespeicherten, dann aktuelle speichern

	mov		XL,XH
	ldi		YL,Low(EE_MaxTemperatur)
	ldi		YH,High(EE_MaxTemperatur)
	rcall	EEPROM_Write
	rjmp	CheckTemperatur_End

	; *******************************************************************************
	; * Lfter Aus Temperatur im grnen Bereich										*
	; *******************************************************************************
CheckTemperatur_LuefterAus:
	lds		XH,RAM_TemperaturSchwellwertLow
	cp		XL,XH	
	brlo	CheckTemperatur_LuefterNichtAus

	cbr		R_OUTPUT,K_Fan
	cbr		R_STATUS,(1<<Flag_TempWarning)

CheckTemperatur_LuefterNichtAus:
CheckTemperatur_End:
	ret

; ***********************************************************************************
; * Check AKKU Spannung																*
; ***********************************************************************************
CheckAkkuSpannung:
	ldi		XL,AD_AKKU											; Wahl Analog Channel 
	rcall	Read_Analog

	; *******************************************************************************
	; * Akku Spannung Entladeschutz - Flag setzen	 								*
	; *******************************************************************************
	lds		XH,RAM_AkkuSchwellwertLowEnergyYellow
	cp		XL,XH
	brsh	CheckAkkuSpannung_Erholt																
	sbr		R_STATUS,(1<<Flag_AkkuWarning)						; Flag setzen -> AKKU schwach Warnen	
	cbr		R_STATUS,(1<<Flag_AkkuLock)							; Flag lschen

	lds		XH,RAM_AkkuSchwellwertLowEnergyRed
	cp		XL,XH
	brsh	CheckAkkuSpannung_Erholt

	; *******************************************************************************
	; * Akku Spannung Entladeschutz - Flag setzen	 								*
	; *******************************************************************************
	cbr		R_STATUS,(1<<Flag_AkkuWarning)						; Flag lschen
	sbr		R_STATUS,(1<<Flag_AkkuLock)							; Flag setzen -> AKKU sehr schwach keine Leistugsverbraucher mehr zulassen
	lds		XH,RAM_AkkuSchwellwertNoEnergy
	cp		XL,XH	
	brsh	CheckAkkuSpannung_End	

	; *******************************************************************************
	; * Akku Tiefentladen Wert speichern wenn tiefer als im EEprom und alles aus	*
	; *******************************************************************************
	mov		XH,XL
	ldi		YL,Low(EE_MinAkkuSpannung)
	ldi		YH,High(EE_MinAkkuSpannung)
	rcall	EEPROM_Read
	cp		XH,XL
	brsh	CheckAkkuSpannung_Deaktivieren
	
	; *******************************************************************************
	; * Akku Spannung speichern														*
	; *******************************************************************************
	mov		XL,XH
	ldi		YL,Low(EE_MinAkkuSpannung)
	ldi		YH,High(EE_MinAkkuSpannung)
	rcall	EEPROM_Write
	brsh	CheckAkkuSpannung_Deaktivieren

CheckAkkuSpannung_Erholt:
	lds		XH,RAM_AkkuSchwellwertHighEnergy
	cp		XL,XH	
	brlo	CheckAkkuSpannung_AkkuNichtErholt

	cbr		R_STATUS,(1<<Flag_AkkuWarning)						; Flag lschen
	cbr		R_STATUS,(1<<Flag_AkkuLock)							; Flag lschen
	rjmp	CheckAkkuSpannung_End

CheckAkkuSpannung_Deaktivieren:
	cbi		OnOffPort,OnOff

CheckAkkuSpannung_AkkuNichtErholt:
CheckAkkuSpannung_End:
	ret

; ***********************************************************************************
; * Werte ber RS232 senden	(512 Bytes)												*
; * Wird nur ausgefhrt wenn Daten ber die RS232 abgerufen werden 					*
; ***********************************************************************************
SendData:
	sbrs	R_STATUSRS232,Flag_SetData
	rjmp	SendDataEnd

	cli															; alle Interrupts Disabled

	; *******************************************************************************
	; * Aktuelle Temperatur einlesen und speichern									*
	; *******************************************************************************
	ldi		XL,AD_AKKU											; Wahl Analog Channel 
	rcall	Read_Analog
	ldi		YL,Low(EE_AktuelleAKKUSpannung)						; Aktuelle AKKU Spannung speichern
	ldi		YH,High(EE_AktuelleAKKUSpannung)
	rcall	EEPROM_Write

	; *******************************************************************************
	; * Aktuelle AKKU Spannung einlesen und speichern								*
	; *******************************************************************************
	ldi		XL,AD_Temperatur									; Wahl Analog Channel 
	rcall	Read_Analog
	ldi		YL,Low(EE_AktuelleTemperatur)						; Aktuelle Temperatur speichern
	ldi		YH,High(EE_AktuelleTemperatur)
	rcall	EEPROM_Write

	; *******************************************************************************
	; * Aktuellen Port Status einlesen und speichern								*
	; *******************************************************************************
	mov		XL,R_OUTPUT
	ldi		YL,Low(EE_PortStatus)
	ldi		YH,High(EE_PortStatus)
	rcall	EEPROM_Write

	clr		YH

SendDataLoop1:
	clr		YL

SendDataLoop2:
	wdr
	rcall	EEPROM_Read

SendDataWaitForFreeBus:
	sbis	UCSRA,UDRE											; Warte auf freien Bus
	rjmp	SendDataWaitForFreeBus

	rcall	ToggleStatusLEDYellow
	out		UDR,XL	
	inc		YL	
	cpi		YL,0
	brne	SendDataLoop2

	inc		YH
	cpi		YH,2
	brne	SendDataLoop1

	sbi		StatusLEDYellowPort,StatusLEDYellow

SendDataEnd:
	cbr		R_STATUSRS232,(1<<Flag_SetData) 
	sei															; alle Interruppts Enabled	
	ret

; ***********************************************************************************
; * Eingegangene Daten im EEprom speichern (32 Bytes)								*
; * Wird nur ausgefhrt wenn das SaveData Flag gesetzt ist, also wenn Daten ber 	*
; * die RS232 eingetroffen sind														*
; ***********************************************************************************
SaveDataToEEprom:
	sbrs	R_STATUSRS232,Flag_SaveData							; Warten bis alle Werte der UART im SRAM
	rjmp	SaveDataToEEprom_End
			
	; *********************************** 
	; * Werte SRAM ins Eprom speichern	*
	; *********************************** 
	cli															; alle Interrupts Disabled
	ldi		ZL,Low(RAM_DatenBuffer)
	ldi 	ZH,High(RAM_DatenBuffer)
	ldi		YL,Low(EE_RWDaten)
	ldi		YH,High(EE_RWDaten)

SaveDataToEEprom_Loop:
	wdr
	rcall	ToggleStatusLEDRed
	ld		XL,Z+
	rcall	EEPROM_Write
	adiw	YH:YL,1
	ldi		XH,High(RAM_DatenBuffer+K_DatenAnzahlSchreiben)
	cpi		ZL,Low(RAM_DatenBuffer+K_DatenAnzahlSchreiben)
	cpc		ZH,XH
	brne	SaveDataToEEprom_Loop

	cbr		R_STATUSRS232,(1<<Flag_SaveData)
	sei															; alle Interruppts Enabled

SaveDataToEEprom_WaitForFreeBus:
	sbis	UCSRA,UDRE											; Warte auf freien Bus
	rjmp	SaveDataToEEprom_WaitForFreeBus
	
	ldi		XL,'A'
	out		UDR,XL
	rcall	EEpromToRAM											; Daten aus dem EEprom ins RAM holen
	sbi		StatusLEDRedPort,StatusLEDRed

SaveDataToEEprom_End:
	ret

; ***********************************************************************************
; * Interupt Einsprnge																*
; ***********************************************************************************
EXT_INT0:
	push	XH
	in		XH,SREG
	push	XL
	push	XH
	push	YL
	push	YH
	push	ZL
	Push	ZH

	in 		XL,TCNT1L											; Read TCNT1 into XL:XH
	in		XH,TCNT1H
	sbic	DataInPort,DataIn
	rjmp	EXT_INT0_ToRising

;EXT_INT0_ToFalling:
	mov		R_FallingTimeL,XL
	mov		R_FallingTimeH,XH
	rjmp	EXT_INT0_End

EXT_INT0_ToRising:
	; *******************************************************************************
	; * Rechnung XH:XL (16Bit) - falling_time (16Bit)								*
	; *******************************************************************************
	sub		XL,R_FallingTimeL
	sbc		XH,R_FallingTimeH
	; *******************************************************************************
	; * Ende Rechnung																*
	; *******************************************************************************

	ldi		YH,High(K_StartZeit/K_FaktorTimer)	
	cpi		XL,Low(K_StartZeit/K_FaktorTimer)
	cpc		XH,YH
	brlo	EXT_INT0_IstBit

	; *******************************************************************************
	; * Rechnung Dataschwelle (16Bit) - #(Korrekturfaktor/Faktortimer) (16Bit)		*
	; *******************************************************************************
	subi	XL,Low(K_StartZeit/K_FaktorTimer)
	sbci	XH,High(K_StartZeit/K_FaktorTimer)

	ldi		YH,High(K_DataOffsetKorrekturBegrenzung/K_FaktorTimer)
	cpi		XL,Low(K_DataOffsetKorrekturBegrenzung/K_FaktorTimer)
	cpc		XH,YH
	brlo	EXT_INT0_DataOffsetKorrekturUebernehmen				; Korrekturwert begrenzen

	ldi		XL,Low(K_DataOffsetKorrekturBegrenzung/K_FaktorTimer)
	ldi		XH,High(K_DataOffsetKorrekturBegrenzung/K_FaktorTimer)

EXT_INT0_DataOffsetKorrekturUebernehmen:
	; *******************************************************************************
	; * Rechnung Dataschwelle (16Bit) + #(Korrekturfaktor/Faktortimer) (16Bit)		*
	; *******************************************************************************
	ldi		R_DataSchwelleL,low(K_BitZeit/K_FaktorTimer)
	ldi		R_DataSchwelleH,high(K_BitZeit/K_FaktorTimer)
	add		R_DataSchwelleL,XL
	adc		R_DataSchwelleH,XH

	ldi		XL,K_BitAnzahl
	mov		R_BitZaehler,XL
	clr		R_DataBufferLow
	clr		R_DataBufferHigh

	rjmp	EXT_INT0_End

EXT_INT0_IstBit:
	cp		XL,R_DataSchwelleL
	cpc		XH,R_DataSchwelleH

	rol		R_DataBufferLow
	rol		R_DataBufferHigh

	dec		R_BitZaehler
	brne	EXT_INT0_End

	com		R_DataBufferLow
	com		R_DataBufferHigh

	; *******************************************************************************
	; * jetzt stehen die Daten in XL/XH												*
	; *******************************************************************************
	lds		XH,RAM_HerstellerCode
	mov		XL,R_DataBufferHigh
	cp		XL,XH												; HerstellerCode prfen				
	brne	EXT_INT0_End
	ldi		XH,8

	ldi		ZL,Low(RAM_FunkCodeTaste1)
	ldi		ZH,High(RAM_FunkCodeTaste1)


EXT_INT0_IstBitLoop:	
	ld		XL,Z+
	cp		XL,R_DataBufferLow	
	breq	EXT_INT0_IstTaste
		
	dec		XH
	brne	EXT_INT0_IstBitLoop

	rjmp	EXT_INT0_End

EXT_INT0_IstTaste:
	sbrc	R_STATUS,Flag_NeueTasteFunk
	rjmp	EXT_INT0_End
	sts		RAM_TastenBufferNeu,R_DataBufferLow					; <- DataBuffer Low bergeben
	sbr		R_STATUS,(1<<Flag_NeueTasteFunk) 					; set Flag

EXT_INT0_End:
	pop		ZH
	pop		ZL
	pop		YH
	pop		YL
	pop		XH
	pop		XL
	out		SREG,XH
	pop		XH
	reti

; ***********************************************************************************
; * Interrupt fr 100ms Task														*
; ***********************************************************************************
TIM2_COMP:
	push	XL
	push	XH
	in		XH,SREG												; store SREG

	in		XL,OCR2
	subi	XL,256-K_Intervall10ms			
	out		OCR2,XL												; nchste Compare festlegen
	sbr		R_STATUS,(1<<Flag_10ms) 							; set Flag

TIM2_COMP_End:
	out		SREG,XH
	pop		XH
	pop		XL
	reti

; ***********************************************************************************
; * Interrupt fr USART RxC															*
; ***********************************************************************************
USART_RXC:
	push 	XL           										; XL auf dem Stack sichern
	in 		XL,SREG												; Einlesen des SREG
	push	XL
	push	XH
	push	YL
	push	YH
	push	ZL
	push	ZH
	in		XL,UDR

	; *********************************** 
	; * Auswertung						*
	; *********************************** 
	sbrc	R_STATUSRS232,Flag_Remote
	rjmp	USART_RXC_Remote

	sbrc	R_STATUSRS232,Flag_GetData
	rjmp	USART_RXC_DataIn

	cpi		XL,'R'
	brne	USART_RXC_NotRemote									; Remote

	sbr		R_STATUSRS232,(1<<Flag_Remote) 
	rjmp	USART_RXC_End

USART_RXC_NotRemote:
	cpi		XL,'G'												; Get Data
	brne	USART_RXC_NotGetData

	sbr		R_STATUSRS232,(1<<Flag_SetData) 
	rjmp	USART_RXC_End

USART_RXC_NotGetData:
	cpi		XL,'S'
	brne	USART_RXC_NotSetData

	ldi		XL,Low(RAM_DatenBuffer)
	sts		RAM_DatenBufferZeiger,XL
	ldi		XL,High(RAM_DatenBuffer)
	sts 	RAM_DatenBufferZeiger+1,XL
	sbr		R_STATUSRS232,(1<<Flag_GetData)
	rjmp	USART_RXC_End

USART_RXC_NotSetData:
	cpi		XL,'D'
	brne	USART_RXC_NotDeleteData

	sbr		R_STATUSRS232,(1<<Flag_DeleteData) 
	rjmp	USART_RXC_End

USART_RXC_NotDeleteData:
	rjmp	USART_RXC_End

USART_RXC_Remote:
	eor		R_OUTPUT,XL										; <- DataBuffer Low bergeben
	
	sbrs	R_OUTPUT,Flag_Fan
	cbr		R_STATUSRS232,(1<<Flag_FanControlInaktive)

	sbrc	R_OUTPUT,Flag_Fan
	sbr		R_STATUSRS232,(1<<Flag_FanControlInaktive)

	rcall	SetOutput
	cbr		R_STATUSRS232,(1<<Flag_Remote) 
	rjmp	USART_RXC_End

USART_RXC_DataIn:
	lds		ZL,RAM_DatenBufferZeiger
	lds		ZH,RAM_DatenBufferZeiger+1
	st		Z+,XL
	sts		RAM_DatenBufferZeiger,ZL
	sts		RAM_DatenBufferZeiger+1,ZH
	ldi		YH,High(RAM_DatenBuffer+K_DatenAnzahlSchreiben)
	cpi		ZL,Low(RAM_DatenBuffer+K_DatenAnzahlSchreiben)
	cpc		ZH,YH
	brne	USART_RXC_End

	cbr		R_STATUSRS232,(1<<Flag_GetData)
	sbr		R_STATUSRS232,(1<<Flag_SaveData)

USART_RXC_End:
	pop		ZH
	pop		ZL
	pop		YH
	pop		YL
	pop		XH
	pop		XL												; LESEN von SREG vom STACK (KOPIE)
	out		SREG,XL											; Wiederherstellen von SREG
	pop 	XL      										; XL wiederherstellen
	reti

;EXT_INT0: 													; IRQ0 Handler
EXT_INT1: 													; IRQ1 Handler
;TIM2_COMP: 												; Timer2 Compare Handler
TIM2_OVF: 													; Timer2 Overflow Handler
TIM1_CAPT:													; Timer1 Capture Handler
TIM1_COMPA_INT:												; Timer1 Compare A Handler
TIM1_COMPB_INT:												; Timer1 Compare B Handler
TIM1_OVF: 													; Timer1 Overflow Handler
TIM0_OVF: 													; Timer0 Overflow Handler
SPI_STC: 													; SPI Transfer Complete Handler
;USART_RXC: 												; USART, RX Complete Handler
USART_UDRE: 												; USART, UDR Empty Handler
USART_TXC: 													; USART, TX Complete Handler
ADC_INT: 													; ADC Conversion Complete Handler
EE_RDY: 													; EEPROM Ready Handler
ANA_COMP: 													; Analog Comparator Handler
TWI: 														; 2-wire Serial Interface Handler
EXT_INT2:													; IRQ2 Handler							(Mega16)
TIM0_COMP:													; Timer0 Compare Handler				(Mega16)
SPM_RDY: 													; Store Program Memory Ready Handler
	reti;

; ***********************************************************************************
; * Include Files																	*
; ***********************************************************************************
.include "LibV01.asm"
